# coding=utf8
from typing import Union
from datetime import datetime


def hex_str(byte_string, splitter='') -> str:
    r"""
    >>> dec_str('\x29\x4A\x00\x31', ' ')
    '29 4A 00 31'
    """
    return splitter.join('%02X' % byte for byte in byte_string)


def dec_str(byte_string, splitter='') -> str:
    r"""
    >>> dec_str('\x29\x4A\x00\x31', ' ')
    '41 74 00 49'
    """
    return splitter.join('%02d' % byte for byte in byte_string)


def to_datetime(from_string: Union[str, bytes], from_format, to_format) -> str:
    r"""
    >>> to_datetime('160123', '%d%m%y', '%Y-%m-%d')
    '2023-01-16'
    >>> to_datetime(b'\x16\x01\x23', '%d%m%y', '%Y-%m-%d')
    '2023-01-16'
    >>> to_datetime(b'\x13\x57\x43\x16\x01\x23', '%H%M%S%d%m%y', '%Y-%m-%d %H:%M:%S')
    '2023-01-16 13:57:43'
    """
    if isinstance(from_string, bytes):
        from_string = hex_str(from_string)
    parsed_datetime = datetime.strptime(from_string, from_format)
    return parsed_datetime.strftime(to_format)


def chunk_string(string, length) -> iter:
    return (string[0 + i:length + i] for i in range(0, len(string), length))



# coding=utf8
from mercury_base.utils import dec_str, to_datetime


def get_serial_number_and_date_of_manufacture(meter) -> dict:
    data = meter.send_command(0x08, 0x00)
    return {
        'serial_number': int(dec_str(data[:4])),
        'date_of_manufacture': to_datetime(dec_str(data[4:7]), '%d%m%y', '%Y-%m-%d'),
    }


def get_passport(meter) -> dict:
    data = meter.send_command(0x08, 0x0100)
    result = {
        'serial_number': int(dec_str(data[:4])),
        'date_of_manufacture': to_datetime(dec_str(data[4:7]), '%d%m%y', '%Y-%m-%d'),
        'firmware_version': '.'.join('%d' % b for b in data[7:10]),
        'meter_version': '.'.join('%d' % b for b in data[18:20]),
    }
    result.update(_parse_version(data[10:16]))
    return result


def _parse_version(data) -> dict:
    return {}


def get_transformation_ratios(meter) -> dict:
    data = meter.send_command(0x08, 0x02)
    return {
        'voltage': int(dec_str(data[:2])),
        'current': int(dec_str(data[2:4])),
    }


def get_firmware_version(meter) -> str:
    data = meter.send_command(0x08, 0x03)
    return '.'.join('%d' % b for b in data)


def get_additional_timeout_multiplier(meter) -> int:
    data = meter.send_command(0x08, 0x04)
    return int(dec_str(data))


def get_main_timeout_multiplier(meter) -> int:
    data = meter.send_command(0x08, 0x1D)
    return int(dec_str(data))


def get_info(meter) -> dict:
    model = 'unknown'
    features = []
    data = meter.send_command(0x08, 0x12, 0)
    """ TODO: parse answer """
    return {
        'model': model,
        'features': features,
    }
